import * as mc from "@minecraft/server";

const THE_DIVINE_SEAL_ID = "thelake:the_divine_seal";
const MIDAS_RESPITE_PEARL_ID = "thelake:midas_respite_pearl";
const MESSAGE_DURATION = 60; // 3 seconds (20 ticks per second)
const RESPITE_DURATION = 200; // 10 seconds (20 ticks per second)

mc.world.afterEvents.playerSpawn.subscribe((event) => {
    if (event.initialSpawn) {
        const player = event.player;
        mc.system.runTimeout(() => {
            player.runCommand('tellraw @s {"rawtext":[{"text":"§6⚔ §eThe Divine Touch §6⚔\n§fYou have been chosen by the gods.\n§eYour touch shall turn all to gold."}]}');
            player.runCommand('playsound mob.wither.death @s ~~~ 1 1 1');
        }, 40);
    }
});

mc.world.afterEvents.itemCompleteUse.subscribe((event) => {
    const player = event.source;
    const item = event.itemStack;

    if (item.typeId === MIDAS_RESPITE_PEARL_ID) {
        if (!player.hasTag("midas_respite")) {
            player.addTag("midas_respite");
            showTemporaryMessage(player, "§e≫ §eMidas Sealed for 10 seconds §e≪");
            player.runCommand('playsound random.levelup @s ~~~ 1 1 1');

            startCountdown(player);

            mc.system.runTimeout(() => {
                player.removeTag("midas_respite");
                showTemporaryMessage(player, "§6≫ §6Midas Unsealed §6≪");
                player.runCommand('playsound random.orb @s ~~~ 1 1 1');
            }, RESPITE_DURATION);
        }
    }
});

function startCountdown(player) {
    let countdown = 9;
    const countdownInterval = mc.system.runInterval(() => {
        if (countdown > 0) {
            player.onScreenDisplay.setActionBar(`§e≫ §eMidas Respite: §f${countdown} §eseconds left §e≪`);
            countdown--;
        } else {
            mc.system.clearRun(countdownInterval);
        }
    }, 20); // Run every second (20 ticks)
}

mc.system.runInterval(() => {
    for (const player of mc.world.getAllPlayers()) {
        const isMidasActive = !isTheDivineSealEquipped(player) && !player.hasTag("midas_respite");
        if (isMidasActive) {
            transformBlocksAroundPlayer(player);
        }
    }
}, 1);

function isTheDivineSealEquipped(player) {
    const inventory = player.getComponent("inventory").container;
    const firstSlotItem = inventory.getItem(0);

    if (firstSlotItem && firstSlotItem.typeId === THE_DIVINE_SEAL_ID) {
        if (!player.hasTag("midas_sealed")) {
            showTemporaryMessage(player, "§c≫ §cMidas Sealed §c≪");
            player.addTag("midas_sealed");
            mc.system.runTimeout(() => {
                player.runCommand('playsound random.anvil_land @s ~~~ 1 1 1');
            }, 5);
        }
        return true;
    } else {
        if (player.hasTag("midas_sealed")) {
            showTemporaryMessage(player, "§6≫ §6Midas Unsealed §6≪");
            player.removeTag("midas_sealed");
            player.runCommand('playsound random.orb @s ~~~ 1 1 1');
        }
        return false;
    }
}

function showTemporaryMessage(player, message) {
    player.onScreenDisplay.setActionBar(message);
    mc.system.runTimeout(() => {
        player.onScreenDisplay.setActionBar("");
    }, MESSAGE_DURATION);
}

function transformBlocksAroundPlayer(player) {
    const playerPosition = player.location;
    for (let y = -1; y <= 1; y++) {
        const blockPos = {
            x: Math.floor(playerPosition.x),
            y: Math.floor(playerPosition.y) + y,
            z: Math.floor(playerPosition.z)
        };
        transformToGold(player, blockPos);
    }
}

function transformToGold(player, blockPos) {
    const block = player.dimension.getBlock(blockPos);
    if (block && shouldTransformToGold(block)) {
        block.setPermutation(mc.BlockPermutation.resolve("minecraft:gold_block"));
        spawnGoldParticles(player, blockPos);
    }
}

function shouldTransformToGold(block) {
    const nonTransformableBlockIds = ["minecraft:air", "minecraft:gold_block", "minecraft:bedrock"];
    return !nonTransformableBlockIds.includes(block.typeId);
}

function spawnGoldParticles(player, blockPos) {
    const particleEffects = [
        "minecraft:totem_particle",
        "minecraft:enchanting_table_particle",
        "minecraft:villager_happy",
        "minecraft:basic_flame_particle"
    ];

    for (const effect of particleEffects) {
        player.dimension.spawnParticle(effect, 
            { x: blockPos.x + 0.5, y: blockPos.y + 1, z: blockPos.z + 0.5 },
            new mc.MolangVariableMap()
        );
    }
}